/**
 ** A Virtual Host Monitor
 **
 ** Copyright 2009 Gary Wuertz gary@issiweb.com
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef TOPVHOST_H
#define TOPVHOST_H

#include "config.h"
#include "monitor.h"
/**
 * Application options
 */
typedef struct {
  char*  configFile;      // configuration file
  char*  configSection;   // config file section
  bool   diagnose;        // diagnose settings
  bool   init;            // read logs at startup
  bool   verify;          // verify settings
} appOptions;
/**
 * Field format specification
 */
typedef struct {
  int                       field;            // format character
  std::string               format;           // format
} fSpec;
/**
 * The application
 */
class topvhost {
public:
  enum {                      // display sort order
    BY_ACCESS,
    BY_HITS,
    BY_NAME
  };
  appOptions                options;          // Application options
  Config*                   conf;             // Configuration
  std::string               logfmt;           // log format
  std::vector<fSpec>        hdr;              // header spec
  std::vector<fSpec>        row;              // row spec
  std::vector<std::string>  needs;            // fields needed for output
  std::vector<std::string>  gets;             // fields provided by log record
  std::map<int,int>         xfer;             // map 'gets' to 'needs'
  bool                      debug;            // for diganostics
  bool                      paint;            // forced repaint
  bool                      resize;           // resize the window
  int                       exit;             // exit code
  int                       maxx;             // number of screen columns
  int                       maxy;             // number of screen rows
  int                       namelen;          // length of longest domain name
  unsigned                  nidx;             // location of domain in output params
  int                       order;            // current display order
  int                       passes;           // number of execution loops
  int                       spos;             // scroll position
  int                       ssz;              // scroll size
  int                       sunit;            // number of lines per row
  time_t                    start;            // time the application started
  int                       total;            // total hosts
  void*                     window;           // hide ncurses from rest of app

  void    begin(int n, int m);                            // start the application
  void    diagnose();                                     // diagnositics
  void    end(void);                                      // stop the application
  int     input(void);                                    // get app input
  void    init(appOptions options);                       // initialize the app
  void    parseSpec(std::vector<fSpec>* v,const char *n); // parse format spec
  void    wmove(int y, int x);                            // move cursor
  void    wprint(std::string fmt,float v);                // print arg
  void    wprint(std::string fmt,int v);                  // print arg
  void    wprint(std::string fmt,long v);                 // print arg
  void    wput(int ch);                                   // add character
  void    wputs(const char *);                            // add string
  int     wx(void);                                       // get x position
  int     wy(void);                                       // get y position
  void    wyx(void);                                      // get screen size
};
#endif
